#!/usr/bin/env python
# -*- coding: utf-8 -*-

import socket, errno

class Client:
    """For a standard of how to interact with clients (this is not
    used to create client protocols"""
    host = unicode()
    port = int()
    delimiter = '\n'
    buffer = str()
    online = False
    net_sock = None
    
    def __init__( self, socket_struct, server, delimiter = '\n' ):
        """server in this case is the server object creating us"""
        self.net_sock = socket_struct[0]
        self.host = socket_struct[1][0]
        self.port = socket_struct[1][1]
        self.delimiter = delimiter

        self.net_sock.setblocking( False )        
        self.online = True
        self.buffer = str()
        
    def write( self, data ):
        self.net_sock.sendall( data + self.delimiter )

    def read( self ):
        self.buffer += self.net_sock.recv( 4096 )

        if self.buffer.count( self.delimiter ):
            line, self.buffer = self.buffer.split( self.delimiter, 1 )
            return line
    
    def close( self ):
        """Closes the socket"""
        try:
            self.net_sock.shutdown( socket.SHUT_RDWR )
            self.net_sock.close()
        except:
            # not that bad if we pass it...
            pass
        
        self.online = False


class TCP_Server:
    """Basic network(server) event generator, sender"""
    host = unicode()
    port = int()
    delimiter = '\n'

    net_sock = None
    clients = []
    
    def __init__( self, host, port, delimiter = '\n' ):
        """Connect etc"""
        self.host = host
        self.port = port
        self.delimiter = delimiter
        
        self.net_sock = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
        
        self.net_sock.setsockopt( socket.SOL_SOCKET,
                                  socket.SO_REUSEADDR, True )
        self.net_sock.bind( ( self.host, self.port ) )
        self.net_sock.listen( True )

        self.net_sock.setblocking( False ) # non-blocking =)
            
    def close( self ):
        """Disconnect/Close the connection"""
        if type( self.net_sock ) != None:

            # close all clients
            while self.clients:
                self.clients[0].close()
                del self.clients[0]
           
            self.net_sock.close()
            self.net_sock = None

    def close_client( self, client ):
        """Shutdowns and closes the clients connection, where client
        is a conn/client object"""
        naughty_client = filter( lambda c: c == client, self.clients )

        self.on_disconnect( client )
        
        if naughty_client != []:
            naughty_client[0].close()
            
        self.clients.remove( client )
        
    def poll( self ):
        """If there is any thing to do send it to handel"""
        # Try to accept new clients
        try:
            self.clients.append( Client( self.net_sock.accept(), self,
                                         self.delimiter ) )
            self.on_connect( self.clients[-1] )
        except socket.error, why:
            if why[0] == errno.EWOULDBLOCK:
                pass
            else:
                raise socket.error, why

        # Look for new data
        for x in range( 0, len(  self.clients ) ):
            try:
                client = self.clients[x]
                data = client.read()

                if not data:
                    client.close()
                    self.on_disconnect( client )
                    del self.clients[x]
                else:
                    self.handle( client, data )
                
            except socket.error, why:
                if why[0] in [ errno.ECONNRESET, errno.ENOTCONN,
                               errno.ETIMEDOUT ]:
                    client.close()
                    self.on_disconnect( client )
                    del self.clients[x]
                                        
                elif why[0] in [ 11, errno.EBADF ]:
                    pass
                
                else:
                    raise socket.error, why

            except IndexError, why:
                # This is coused by client disconnecting mid poll
                self.poll()

    def on_connect( self, client ):
        """On connect (new client)"""
        pass

    def on_disconnect( self, client ):
        """Called when a client disconnected"""
        pass

    def handle( self, client, data ):
        """Called when we got data from a client"""
        pass

# ---- Clients
class TCP_Client:
    host = unicode()
    port = int()
    delimiter = '\n'
    buffer = "" # doesn't call handel before there is atleast one delimiter here
    net_sock = None
    connected = False
    
    def __init__( self, host, port, delimiter = '\n' ):
        self.host = host
        self.port = port
        self.buffer = str()

        self.delimiter = delimiter
        self.net_sock = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
        self.net_sock.connect( ( host, port ) )
        self.net_sock.setblocking( False )

        self.connected = True
        self.on_connect()
        
    def close( self ):
        """Closes the connection"""
        if self.net_sock != None:
            self.net_sock.close()
            self.connected = False

    def poll( self ):
        """Tries to poll new data, if none, then no hard feelings"""
        try:
            self.buffer += self.net_sock.recv( 4096 )
            
            if not self.connected:
                self.connected = True

            if self.buffer.count( self.delimiter ):
                for line in self.buffer.split( self.delimiter ):
                    if line != '':
                        self.handle( line )
                self.buffer = ""
        
        except socket.error, why:
            if why[0] in [ errno.ECONNRESET, errno.ENOTCONN ]:
                self.net_sock.close()
                self.on_disconnect()
            elif why[0] in [ 11 ]: # Not connected yet..
                pass
            else:
                raise socket.error, why

    def write( self, data ):
        """Send data to host"""
        self.net_sock.sendall( data + self.delimiter )

    def on_connect( self ):
        """Successfully connected to host"""
        # Not sure about this one..
        pass
    
    def on_disconnect( self ):
        """Lost connection"""
        pass

    def handle( self, data ):
        """Recived some data!"""
        pass


class SSL_TCP_Client( TCP_Client ):
    """A SSL layered client, currently not able to check certificate,
    more then issuer and server string, (object.ssl_issuer,
    object.ssl_server)"""
    net_sock = None
    ssl_sock = None
    ssl_issuer = ""
    ssl_server = ""
    
    def __init__( self, host, port, delimiter = '\n' ):
        self.host = host
        self.port = port
        self.delimiter = delimiter

        self.buffer = str()
        
        self.net_sock = socket.socket( socket.AF_INET, socket.SOCK_STREAM )
        self.net_sock.connect( ( host, port ) )
        self.ssl_sock = socket.ssl( self.net_sock )

        self.ssl_issuer = self.ssl_sock.issuer()
        self.ssl_server = self.ssl_sock.server()
        
        self.net_sock.setblocking( False )

        self.connected = True
        self.on_connect()
        
    def close( self ):
        """Closes the connection"""
        if self.net_sock != None:
            del self.ssl_sock
            self.ssl_sock = None
            self.net_sock.close()
            self.connected = False

    def poll( self ):
        """Tries to poll new data, if none, then no hard feelings"""
        if self.ssl_sock == None:
            return
        
        try:
            self.buffer += self.ssl_sock.read()
            
            if not self.connected:
                self.connected = True

            if self.buffer.count( self.delimiter ):
                for line in self.buffer.split( self.delimiter ):
                    if line != '':
                        self.handle( line )
                self.buffer = ""

        except socket.sslerror, why:
            #print "why: %d; '%s'" %( why[0], why[1] )

            if why[0] in [ 2 ]:
                # This is the unharmfull ones :)
                # 2: The operation did not complete
                pass
            
            elif why[0] in [ 6 ]:
                # Nasties, or just disconnected ;)
                # 6: TLS/SSL connection has been closed
                self.on_disconnect()
                self.close()
            else:
                raise socket.sslerror, why

    def write( self, data ):
        """Send data to host"""
        # If we get an network related error here, maybe on_disconnect
        # should be called
        self.ssl_sock.write( data + self.delimiter )
