#!/usr/bin/env python
# -*- coding: utf-8 -*-

from blamlib.network import TCP_Client

class Socktroll_Client( TCP_Client ):
    """A client/bot protocol against socktroll, not 100% support yet but close,
    This client caches authenticated users with users."""
    logged_in = False
    nick = str()
    users = list()
    
    def __init__( self, host, port, nick ):
        TCP_Client.__init__( self, host, port )
        self.nick = nick
        self.users = list()
    
    def on_connect( self ):
        """Successfully connected to host"""
        pass
            
    def on_disconnect( self ):
        """Lost connection"""
        pass
    
    def handle( self, data ):
        """(internal use) recived some data, parse out what it is"""
        type = data.split(' ')[0]
        args = data.strip().split(' ')[1:]

        # Inital greeting
        if not self.logged_in:
            if type == 'msg' and args[0] == 'server':
                print "<server> %s" % " ".join( args[1:] )
                self.write( "nick %s" % self.nick )
            
            elif type == 'no':
                self.on_bad_nick( args[0] )

            elif type == 'ok':
                self.logged_in = True
                self.on_ok_nick()
                
            return
        
        # Logged in stuff
        if type == 'msg':
            if not self.users.count( args[0] ):
                self.users.append( args[0] )
            self.on_msg( args[0], " ".join( args[1:] ) )
        elif type == 'action':
            if not self.users.count( args[0] ):
                self.users.append( args[0] )
            self.on_action( args[0], " ".join( args[1:] ) )
        elif type == '+':
            self.users.append( args[0] )
            self.on_join( args[0] )
        elif type == '-':
            if self.users.count( args[0] ):
                self.users.remove( args[0] )
            self.on_part( args[0] )
        elif type == 'rename':
            if self.users.count( args[0] ):
                self.users.remove( args[0] )
                
            self.users.append( args[1] )
            self.on_rename( args[0], args[1] )

    
    def send_msg( self, text ):
        """Send a message to a room/chat"""
        self.write( "msg %s" % text )

    def send_action( self, action ):
        """Send a action (ig: /me is hacking)"""
        self.write( "action %s" % action )

    # replace for specific usage 
    def on_bad_nick( self, type ):
        """Your authentication failed, other nick tries etc here"""
        pass
    
    def on_ok_nick( self ):
        """Now your authed and in the room/chat"""
        pass

    def on_msg( self, sender, text ):
        """The room/chat reviced a message (not excluding yourself)"""
        pass
    
    def on_action( self, sender, text ):
        """The room/chat reviced a action (not excluding yourself)"""
        pass

    def on_join( self, who ):
        """'who' entered the room/chat"""
        pass
    
    def on_part( self, who ):
        """'who' left the room/chat"""
        pass

    def on_rename( self, was, now_is ):
        """A client have changed his nick from was to now_is"""
        pass
